using System;
using System.Collections;
using System.Diagnostics;

using j4o.lang;
using com.db4o;
using com.db4o.query;
using com.db4o.config;

class Record { 
    internal String strKey;
    internal long   intKey;
};

public class TestIndex { 
     const int nRecords = 100000;

    static public void Main(string[] args) {    
        Query q;
        Configuration conf = Db4o.configure();
        
        conf.objectClass(typeof(Record)).objectField("strKey").indexed(true);
        conf.objectClass(typeof(Record)).objectField("intKey").indexed(true);
        
        conf.weakReferences(false);
        conf.discardFreeSpace(int.MaxValue);
        conf.automaticShutDown(false);
        conf.lockDatabaseFile(false);


        ObjectContainer db = Db4o.openFile("testindex.yap");

        DateTime start = DateTime.Now;
        long key = 1999;
        int i;        
        for (i = 0; i < nRecords; i++) { 
            Record rec = new Record();
            key = (3141592621L*key + 2718281829L) % 1000000007L;
            rec.intKey = key;
            rec.strKey = Convert.ToString(key);
            db.set(rec);
        }
        db.commit();
        Console.WriteLine("Elapsed time for inserting " + nRecords + " records: " 
                           + (DateTime.Now - start) + " milliseconds");

        start = DateTime.Now;
        key = 1999;
        for (i = 0; i < nRecords; i++) { 
            key = (3141592621L*key + 2718281829L) % 1000000007L;
            q = db.query();
            q.constrain(typeof(Record));
            q.descend("intKey").constrain(key);
            Record rec1 = (Record)q.execute().next();
            q = db.query();
            q.constrain(typeof(Record));
            q.descend("strKey").constrain(Convert.ToString(key));
            Record rec2 = (Record)q.execute().next();
            Debug.Assert(rec1 != null && rec1 == rec2);
        }
        Console.WriteLine("Elapsed time for performing " + nRecords*2 + " index searches: " + (DateTime.Now - start));
 

        start = DateTime.Now;

        q = db.query();
        q.constrain(typeof(Record));
        q.descend("intKey").orderAscending();
        ObjectSet objectSet = q.execute();
        
        key = long.MinValue;
        i = 0;
        while(objectSet.hasNext()){
            Record rec = (Record)objectSet.next();
            Debug.Assert(rec.intKey >= key);
            key = rec.intKey;
            i++;
        }
        Debug.Assert(i == nRecords);

        i = 0;
        q = db.query();
        q.constrain(typeof(Record));
        q.descend("strKey").orderAscending();
        objectSet = q.execute();

        String strKey = "";
        while(objectSet.hasNext()){
            Record rec = (Record)objectSet.next();
            Debug.Assert(rec.strKey.CompareTo(strKey) >= 0); 
            strKey = rec.strKey;
            i++;
        }
        Debug.Assert(i == nRecords);
        Console.WriteLine("Elapsed time for iterating through " + nRecords + " records: " + (DateTime.Now - start));
        
        start = DateTime.Now;
        
        q = db.query();
        q.constrain(typeof(Record));
        objectSet = q.execute();

        while(objectSet.hasNext()){
            db.delete(objectSet.next());
        }
        db.commit();
        Console.WriteLine("Elapsed time for deleting " + nRecords + " records: " + (DateTime.Now - start));
        db.close();
    }
}
